@@ -33,9 +33,7 @@
 #include <QCheckBox>
 #include <QGroupBox>
 #include <QProgressBar>
-#include <QStringList>
 #include <KTextEdit>
-#include <KComboBox>
 #include <KLocalizedString>
 #include <KMessageBox>
 
@@ -60,14 +58,14 @@ void HgSyncBaseDialog::setup()
             this, SLOT(slotUpdateBusy(QProcess::ProcessState)));
     connect(&m_main_process, SIGNAL(stateChanged(QProcess::ProcessState)), 
             this, SLOT(slotUpdateBusy(QProcess::ProcessState)));
-    connect(&m_main_process, SIGNAL(finished(int, QProcess::ExitStatus)), 
-            this, SLOT(slotOperationComplete(int, QProcess::ExitStatus)));
-    connect(&m_main_process, SIGNAL(error(QProcess::ProcessError)), 
-            this, SLOT(slotOperationError()));
-    connect(&m_process, SIGNAL(error(QProcess::ProcessError)), 
-            this, SLOT(slotChangesProcessError()));
-    connect(&m_process, SIGNAL(finished(int, QProcess::ExitStatus)), 
-            this, SLOT(slotChangesProcessComplete(int, QProcess::ExitStatus)));
+    connect(&m_main_process, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished),
+            this, &HgSyncBaseDialog::slotOperationComplete);
+    connect(&m_main_process, &QProcess::errorOccurred,
+            this, &HgSyncBaseDialog::slotOperationError);
+    connect(&m_process, &QProcess::errorOccurred,
+            this, &HgSyncBaseDialog::slotOperationError);
+    connect(&m_process, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished),
+            this, &HgSyncBaseDialog::slotChangesProcessComplete);
     connect(this, SIGNAL(finished(int)), this, SLOT(slotWriteBigSize()));
 }
 
@@ -347,5 +345,4 @@ void HgSyncBaseDialog::switchOptionsButton(bool switchOn)
 }
 
 
-#include "syncdialogbase.moc"
 
