@@ -37,8 +37,16 @@
 #include <QStringList>
 #include <QTextStream>
 #include <QVBoxLayout>
+#include <QListWidget>
 #include <KConfigGroup>
 #include <KWindowConfig>
+#include <QWindow>
+#include <QTableWidget>
+#include <QHeaderView>
+
+#include "svncommitdialog.h"
+#include "svnlogdialog.h"
+#include "svncommands.h"
 
 K_PLUGIN_FACTORY(FileViewSvnPluginFactory, registerPlugin<FileViewSvnPlugin>();)
 
@@ -52,6 +60,7 @@ FileViewSvnPlugin::FileViewSvnPlugin(QObject* parent, const QList<QVariant>& arg
     m_addAction(0),
     m_removeAction(0),
     m_showUpdatesAction(0),
+    m_logAction(nullptr),
     m_command(),
     m_arguments(),
     m_errorMsg(),
@@ -79,7 +88,7 @@ FileViewSvnPlugin::FileViewSvnPlugin(QObject* parent, const QList<QVariant>& arg
     m_commitAction->setIcon(QIcon::fromTheme("svn-commit"));
     m_commitAction->setText(i18nc("@item:inmenu", "SVN Commit..."));
     connect(m_commitAction, SIGNAL(triggered()),
-            this, SLOT(commitFiles()));
+            this, SLOT(commitDialog()));
 
     m_addAction = new QAction(this);
     m_addAction->setIcon(QIcon::fromTheme("list-add"));
@@ -108,6 +117,11 @@ FileViewSvnPlugin::FileViewSvnPlugin(QObject* parent, const QList<QVariant>& arg
     connect(this, SIGNAL(setShowUpdatesChecked(bool)),
             m_showUpdatesAction, SLOT(setChecked(bool)));
 
+    m_logAction = new QAction(this);
+    m_logAction->setText(xi18nc("@action:inmenu", "SVN Log..."));
+    connect(m_logAction, &QAction::triggered,
+            this, &FileViewSvnPlugin::logDialog);
+
     connect(&m_process, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished),
             this, &FileViewSvnPlugin::slotOperationCompleted);
     connect(&m_process, &QProcess::errorOccurred,
@@ -159,6 +173,7 @@ bool FileViewSvnPlugin::beginRetrieval(const QString& directory)
             case 'A': version = AddedVersion; break;
             case 'D': version = RemovedVersion; break;
             case 'C': version = ConflictingVersion; break;
+            case '!': version = MissingVersion; break;
             default:
                 if (filePath.contains('*')) {
                     version = UpdateRequiredVersion;
@@ -201,6 +216,7 @@ bool FileViewSvnPlugin::beginRetrieval(const QString& directory)
 
 void FileViewSvnPlugin::endRetrieval()
 {
+    emit versionInfoUpdated();
 }
 
 KVersionControlPlugin::ItemVersion FileViewSvnPlugin::itemVersion(const KFileItem& item) const
@@ -226,7 +242,7 @@ KVersionControlPlugin::ItemVersion FileViewSvnPlugin::itemVersion(const KFileIte
     while (it != m_versionInfoHash.constEnd()) {
         if (it.key().startsWith(itemDir)) {
             const ItemVersion version = m_versionInfoHash.value(it.key());
-            if (version == LocallyModifiedVersion) {
+            if (version == LocallyModifiedVersion || version == AddedVersion || version == RemovedVersion) {
                 return LocallyModifiedVersion;
             }
         }
@@ -263,6 +279,8 @@ QList<QAction*> FileViewSvnPlugin::actions(const KFileItemList& items) const
             switch (version) {
                 case LocallyModifiedVersion:
                 case ConflictingVersion:
+                case AddedVersion:
+                case RemovedVersion:
                     ++editingCount;
                     break;
                 default:
@@ -294,7 +312,7 @@ QList<QAction*> FileViewSvnPlugin::actions(const KFileItemList& items) const
 
 void FileViewSvnPlugin::updateFiles()
 {
-    execSvnCommand("update", QStringList(),
+    execSvnCommand(QLatin1String("update"), QStringList(),
                    i18nc("@info:status", "Updating SVN repository..."),
                    i18nc("@info:status", "Update of SVN repository failed."),
                    i18nc("@info:status", "Updated SVN repository."));
@@ -305,60 +323,66 @@ void FileViewSvnPlugin::showLocalChanges()
     Q_ASSERT(!m_contextDir.isEmpty());
     Q_ASSERT(m_contextItems.isEmpty());
 
-    const QString command = QLatin1String("mkfifo /tmp/fifo; svn diff ") +
-                            KShell::quoteArg(m_contextDir) +
-                            QLatin1String(" > /tmp/fifo & kompare /tmp/fifo; rm /tmp/fifo");
-    KRun::runCommand(command, 0);
-}
+    // This temporary file will be deleted on Dolphin close. We make an assumption:
+    // when the file gets deleted kompare has already loaded it and no longer needs it.
+    const QString tmpFileNameTemplate = QString("%1/%2.XXXXXX").arg(QDir::tempPath(), QDir(m_contextDir).dirName());
+    QTemporaryFile *file = new QTemporaryFile(tmpFileNameTemplate, this);
 
-void FileViewSvnPlugin::commitFiles()
-{
-    QDialog dialog(0, Qt::Dialog);
-
-    QVBoxLayout* boxLayout = new QVBoxLayout(&dialog);
-
-    boxLayout->addWidget(new QLabel(i18nc("@label", "Description:"),
-                                    &dialog));
-    QPlainTextEdit* editor = new QPlainTextEdit(&dialog);
-    boxLayout->addWidget(editor);
-
-    dialog.setWindowTitle(i18nc("@title:window", "SVN Commit"));
-    auto buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel, &dialog);
-    connect(buttonBox, &QDialogButtonBox::accepted, &dialog, &QDialog::accept);
-    connect(buttonBox, &QDialogButtonBox::rejected, &dialog, &QDialog::reject);
-    auto okButton = buttonBox->button(QDialogButtonBox::Ok);
-    okButton->setDefault(true);
-    okButton->setText(i18nc("@action:button", "Commit"));
-    boxLayout->addWidget(buttonBox);
-
-    KConfigGroup dialogConfig(KSharedConfig::openConfig("dolphinrc"),
-                              "SvnCommitDialog");
-    KWindowConfig::restoreWindowSize(dialog.windowHandle(), dialogConfig);
-
-    if (dialog.exec() == QDialog::Accepted) {
-        // Write the commit description into a temporary file, so
-        // that it can be read by the command "svn commit -F". The temporary
-        // file must stay alive until slotOperationCompleted() is invoked and will
-        // be destroyed when the version plugin is destructed.
-        if (!m_tempFile.open())  {
-            emit errorMessage(i18nc("@info:status", "Commit of SVN changes failed."));
-            return;
-        }
-
-        QTextStream out(&m_tempFile);
-        const QString fileName = m_tempFile.fileName();
-        out << editor->toPlainText();
-        m_tempFile.close();
-
-        QStringList arguments;
-        arguments << "-F" << fileName;
-        execSvnCommand("commit", arguments,
-                       i18nc("@info:status", "Committing SVN changes..."),
-                       i18nc("@info:status", "Commit of SVN changes failed."),
-                       i18nc("@info:status", "Committed SVN changes."));
+    if (!file->open()) {
+        emit errorMessage(i18nc("@info:status", "Could not show local SVN changes."));
+        return;
     }
 
-    KWindowConfig::saveWindowSize(dialog.windowHandle(), dialogConfig, KConfigBase::Persistent);
+    QProcess process;
+    process.setStandardOutputFile(file->fileName());
+    process.start(
+        QLatin1String("svn"),
+        QStringList {
+            QLatin1String("diff"),
+            QLatin1String("--git"),
+            m_contextDir
+        }
+    );
+    if (!process.waitForFinished() || process.exitCode() != 0) {
+        emit errorMessage(i18nc("@info:status", "Could not show local SVN changes: svn diff failed."));
+        file->deleteLater();
+        return;
+    }
+
+    const bool started = QProcess::startDetached(
+        QLatin1String("kompare"),
+        QStringList {
+            file->fileName()
+        }
+    );
+    if (!started) {
+        emit errorMessage(i18nc("@info:status", "Could not show local SVN changes: could not start kompare."));
+        file->deleteLater();
+    }
+}
+
+void FileViewSvnPlugin::commitDialog()
+{
+    QStringList context;
+    if (!m_contextDir.isEmpty()) {
+        context << m_contextDir;
+    } else {
+        for (const auto &i : m_contextItems) {
+            context << i.localPath();
+        }
+    }
+
+    SvnCommitDialog *svnCommitDialog = new SvnCommitDialog(&m_versionInfoHash, context);
+
+    connect(this, &FileViewSvnPlugin::versionInfoUpdated, svnCommitDialog, &SvnCommitDialog::refreshChangesList);
+
+    connect(svnCommitDialog, &SvnCommitDialog::revertFiles, this, QOverload<const QStringList&>::of(&FileViewSvnPlugin::revertFiles));
+    connect(svnCommitDialog, &SvnCommitDialog::diffFile, this, QOverload<const QString&>::of(&FileViewSvnPlugin::diffFile));
+    connect(svnCommitDialog, &SvnCommitDialog::addFiles, this, QOverload<const QStringList&>::of(&FileViewSvnPlugin::addFiles));
+    connect(svnCommitDialog, &SvnCommitDialog::commit, this, &FileViewSvnPlugin::commitFiles);
+
+    svnCommitDialog->setAttribute(Qt::WA_DeleteOnClose);
+    svnCommitDialog->show();
 }
 
 void FileViewSvnPlugin::addFiles()
@@ -379,12 +403,32 @@ void FileViewSvnPlugin::removeFiles()
 
 void FileViewSvnPlugin::revertFiles()
 {
-    execSvnCommand(QStringLiteral("revert"), QStringList(),
+    QStringList arguments;
+
+    // If we are reverting a directory let's revert everything in it.
+    if (!m_contextDir.isEmpty()) {
+        arguments << QLatin1String("--depth") << QLatin1String("infinity");
+    }
+
+    execSvnCommand(QStringLiteral("revert"), arguments,
                    i18nc("@info:status", "Reverting files from SVN repository..."),
                    i18nc("@info:status", "Reverting of files from SVN repository failed."),
                    i18nc("@info:status", "Reverted files from SVN repository."));
 }
 
+void FileViewSvnPlugin::logDialog()
+{
+    SvnLogDialog *svnLogDialog = new SvnLogDialog(m_contextDir);
+
+    connect(svnLogDialog, &SvnLogDialog::errorMessage, this, &FileViewSvnPlugin::errorMessage);
+    connect(svnLogDialog, &SvnLogDialog::operationCompletedMessage, this, &FileViewSvnPlugin::operationCompletedMessage);
+    connect(svnLogDialog, &SvnLogDialog::diffAgainstWorkingCopy, this, &FileViewSvnPlugin::diffAgainstWorkingCopy);
+    connect(svnLogDialog, &SvnLogDialog::diffBetweenRevs, this, &FileViewSvnPlugin::diffBetweenRevs);
+
+    svnLogDialog->setAttribute(Qt::WA_DeleteOnClose);
+    svnLogDialog->show();
+}
+
 void FileViewSvnPlugin::slotOperationCompleted(int exitCode, QProcess::ExitStatus exitStatus)
 {
     m_pendingOperation = false;
@@ -418,6 +462,122 @@ void FileViewSvnPlugin::slotShowUpdatesToggled(bool checked)
     emit itemVersionsChanged();
 }
 
+void FileViewSvnPlugin::revertFiles(const QStringList& filesPath)
+{
+    for (const auto &i : qAsConst(filesPath)) {
+        m_contextItems.append( QUrl::fromLocalFile(i) );
+    }
+    m_contextDir.clear();
+
+    execSvnCommand(QLatin1String("revert"), QStringList() << filesPath,
+                   i18nc("@info:status", "Reverting changes to file..."),
+                   i18nc("@info:status", "Revert file failed."),
+                   i18nc("@info:status", "File reverted."));
+}
+
+void FileViewSvnPlugin::diffFile(const QString& filePath)
+{
+    // For a diff we will export last known file local revision from a remote and compare. We will
+    // not use basic SVN action 'svn diff --extensions -U<lines> <fileName>' because we should count
+    // lines or set maximum number for this.
+    // With a maximum number (2147483647) 'svn diff' starts to work slowly.
+
+    diffAgainstWorkingCopy(filePath, SVNCommands::localRevision(filePath));
+}
+
+void FileViewSvnPlugin::diffAgainstWorkingCopy(const QString& localFilePath, ulong rev)
+{
+    QTemporaryFile *file = new QTemporaryFile(this);
+    if (!SVNCommands::exportFile(QUrl::fromLocalFile(localFilePath), rev, file)) {
+        emit errorMessage(i18nc("@info:status", "Could not show local SVN changes for a file: could not get file."));
+        file->deleteLater();
+        return;
+    }
+
+    const bool started = QProcess::startDetached(
+        QLatin1String("kompare"),
+        QStringList {
+            file->fileName(),
+            localFilePath
+        }
+    );
+    if (!started) {
+        emit errorMessage(i18nc("@info:status", "Could not show local SVN changes: could not start kompare."));
+        file->deleteLater();
+    }
+}
+
+void FileViewSvnPlugin::diffBetweenRevs(const QString& remoteFilePath, ulong rev1, ulong rev2)
+{
+    QTemporaryFile *file1 = new QTemporaryFile(this);
+    QTemporaryFile *file2 = new QTemporaryFile(this);
+    if (!SVNCommands::exportFile(QUrl::fromLocalFile(remoteFilePath), rev1, file1)) {
+        emit errorMessage(i18nc("@info:status", "Could not show local SVN changes for a file: could not get file."));
+        file1->deleteLater();
+        return;
+    }
+    if (!SVNCommands::exportFile(QUrl::fromLocalFile(remoteFilePath), rev2, file2)) {
+        emit errorMessage(i18nc("@info:status", "Could not show local SVN changes for a file: could not get file."));
+        file1->deleteLater();
+        file2->deleteLater();
+        return;
+    }
+
+    const bool started = QProcess::startDetached(
+        QLatin1String("kompare"),
+        QStringList {
+            file2->fileName(),
+            file1->fileName()
+        }
+    );
+    if (!started) {
+        emit errorMessage(i18nc("@info:status", "Could not show local SVN changes: could not start kompare."));
+        file1->deleteLater();
+        file2->deleteLater();
+    }
+}
+
+void FileViewSvnPlugin::addFiles(const QStringList& filesPath)
+{
+    for (const auto &i : qAsConst(filesPath)) {
+        m_contextItems.append( QUrl::fromLocalFile(i) );
+    }
+    m_contextDir.clear();
+
+    addFiles();
+}
+
+void FileViewSvnPlugin::commitFiles(const QStringList& context, const QString& msg)
+{
+    // Write the commit description into a temporary file, so
+    // that it can be read by the command "svn commit -F". The temporary
+    // file must stay alive until slotOperationCompleted() is invoked and will
+    // be destroyed when the version plugin is destructed.
+    if (!m_tempFile.open())  {
+        emit errorMessage(i18nc("@info:status", "Commit of SVN changes failed."));
+        return;
+    }
+
+    QTextStream out(&m_tempFile);
+    const QString fileName = m_tempFile.fileName();
+    out << msg;
+    m_tempFile.close();
+
+    QStringList arguments;
+    arguments << context << "-F" << fileName;
+
+    // Lets clear m_contextDir and m_contextItems variables: we will pass everything in arguments.
+    // This is needed because startSvnCommandProcess() uses only one QString for svn transaction at
+    // a time but we want to commit everything.
+    m_contextDir.clear();
+    m_contextItems.clear();
+
+    execSvnCommand(QLatin1String("commit"), arguments,
+                   i18nc("@info:status", "Committing SVN changes..."),
+                   i18nc("@info:status", "Commit of SVN changes failed."),
+                   i18nc("@info:status", "Committed SVN changes."));
+}
+
 void FileViewSvnPlugin::execSvnCommand(const QString& svnCommand,
                                        const QStringList& arguments,
                                        const QString& infoMsg,
@@ -446,10 +606,14 @@ void FileViewSvnPlugin::startSvnCommandProcess()
         arguments << m_contextDir;
         m_contextDir.clear();
     } else {
-        const KFileItem item = m_contextItems.takeLast();
-        arguments << item.localPath();
-        // the remaining items of m_contextItems will be executed
-        // after the process has finished (see slotOperationFinished())
+        // If m_contextDir is empty and m_contextItems is empty then all svn arguments are in
+        // m_arguments (for example see commitFiles()).
+        if (!m_contextItems.isEmpty()) {
+            const KFileItem item = m_contextItems.takeLast();
+            arguments << item.localPath();
+            // the remaining items of m_contextItems will be executed
+            // after the process has finished (see slotOperationFinished())
+        }
     }
     m_process.start(program, arguments);
 }
@@ -470,13 +634,25 @@ QList<QAction*> FileViewSvnPlugin::directoryActions(const KFileItem& directory)
 
     const ItemVersion version = itemVersion(directory);
     m_showLocalChangesAction->setEnabled(enabled && (version != NormalVersion));
-    m_commitAction->setEnabled(enabled && (version == LocallyModifiedVersion));
+    m_addAction->setEnabled(enabled && (version == UnversionedVersion));
+    m_removeAction->setEnabled(enabled && (version == NormalVersion));
+    if (version == LocallyModifiedVersion || version == AddedVersion || version == RemovedVersion) {
+        m_commitAction->setEnabled(enabled);
+        m_revertAction->setEnabled(enabled);
+    } else {
+        m_commitAction->setEnabled(false);
+        m_revertAction->setEnabled(false);
+    }
 
     QList<QAction*> actions;
     actions.append(m_updateAction);
     actions.append(m_showLocalChangesAction);
     actions.append(m_commitAction);
     actions.append(m_showUpdatesAction);
+    actions.append(m_addAction);
+    actions.append(m_removeAction);
+    actions.append(m_revertAction);
+    actions.append(m_logAction);
     return actions;
 }
 
