@@ -47,19 +47,34 @@ public:
 signals:
     /// Invokes m_showUpdatesAction->setChecked(checked) on the UI thread.
     void setShowUpdatesChecked(bool checked);
+
+    /**
+     * Is emitted if current SVN directory status got updated. Not necessarily means
+     * it's changed. Emitted right after #endRetrieval().
+     */
+    void versionInfoUpdated();
+
 private slots:
     void updateFiles();
     void showLocalChanges();
-    void commitFiles();
+    void commitDialog();
     void addFiles();
     void removeFiles();
     void revertFiles();
+    void logDialog();
 
     void slotOperationCompleted(int exitCode, QProcess::ExitStatus exitStatus);
     void slotOperationError();
 
     void slotShowUpdatesToggled(bool checked);
 
+    void revertFiles(const QStringList& filesPath);
+    void diffFile(const QString& filePath);
+    void diffAgainstWorkingCopy(const QString& localFilePath, ulong rev);
+    void diffBetweenRevs(const QString& remoteFilePath, ulong rev1, ulong rev2);
+    void addFiles(const QStringList& filesPath);
+    void commitFiles(const QStringList& context, const QString& msg);
+
 private:
     /**
      * Executes the command "svn {svnCommand}" for the files that have been
@@ -92,6 +107,7 @@ private:
     QAction* m_removeAction;
     QAction* m_revertAction;
     QAction* m_showUpdatesAction;
+    QAction* m_logAction;
 
     QString m_command;
     QStringList m_arguments;
